#include <Arduino.h>
#include <IRremoteESP8266.h>
#include <IRsend.h>
#include <Adafruit_Sensor.h>
#include <DHT.h>
#include <DHT_U.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>

// Update these with values suitable for your network.

const char* ssid = "WiFiName";
const char* password = "WiFiPW";
const char* mqtt_server = ".duckdns.org";
const char* mqttUser = "YourMqttUser";
const char* mqttPassword = "YourMqttUserPassword";

int pirPin = D7;
#define DHTPIN 2     // Digital pin connected to the DHT sensor 
#define DHTTYPE    DHT22     // DHT 22 (AM2302)
DHT_Unified dht(DHTPIN, DHTTYPE);

WiFiClient espClient;
PubSubClient client(espClient);
unsigned long lastMsg = 0;
unsigned long lastTempMsg = 0;
unsigned long lastMoveMsg = 0;
#define MSG_BUFFER_SIZE  (50)
char msg[MSG_BUFFER_SIZE];
int value = 0;
const uint16_t kIrLed = 4;  // ESP8266 GPIO pin to use. Recommended: 4 (D2).
IRsend irsend(kIrLed);

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
  //Serial.print("Message arrived [");
  //Serial.print(topic);
  //Serial.print("] ");
  long a=0;
  for (int i = 0; i < length; i++) {
    //Serial.println(((int)payload[i]-48)); 
    //Serial.println(pow(10,(length-1-i)));
    a+=((int)payload[i]-48)*(pow(10,(length-1-i)));
  }
  //Serial.println(a,HEX);
  if(strcmp (topic,"sendCoolixIR") == 0){
    //Serial.println("sendCoolixIR");
    irsend.sendCOOLIX(a);
  }else if(strcmp (topic,"sendSamsungIR") == 0){
    //Serial.println("sendSamsungIR");
    long b = (0xE0E0 << 16) | a;
    irsend.sendSAMSUNG(b);
  }else if(strcmp (topic,"sendNECIR") == 0){
    //Serial.println("sendNECIR");
    irsend.sendNEC(a);
  }
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Create a random client ID
    String clientId = "ESP8266Client-";
    clientId += String(random(0xffff), HEX);
    // Attempt to connect
    if (client.connect("ESP8266Client", mqttUser, mqttPassword)) {
      Serial.println("connected");
      // Once connected, publish an announcement...
      client.publish("outTopic", "hello world");
      // ... and resubscribe
      client.subscribe("sendCoolixIR");
      client.subscribe("sendSamsungIR");
      client.subscribe("sendNECIR");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}


 
void setup() {
  irsend.begin();
  Serial.begin(115200);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);

  dht.begin();
  sensor_t sensor;
  dht.temperature().getSensor(&sensor);
  dht.humidity().getSensor(&sensor);
}

void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  unsigned long now = millis();
  sensors_event_t event;
  if (now - lastTempMsg > 300000) {
    lastTempMsg = now;
    
    dht.temperature().getEvent(&event);
    if (isnan(event.temperature)) {
      //Serial.println(F("Error reading temperature!"));
    }
    else {
      snprintf (msg, MSG_BUFFER_SIZE, "%.2f", event.temperature);
      client.publish("temp", msg);
      //Serial.print(msg);
      //Serial.println(F("°C"));
    }
    
    // Get humidity event and print its value.
    dht.humidity().getEvent(&event);
    if (isnan(event.relative_humidity)) {
      //Serial.println(F("Error reading humidity!"));
    }
    else {
      snprintf (msg, MSG_BUFFER_SIZE, "%.2f", event.relative_humidity);
      client.publish("humidity", msg);
     // Serial.print(msg);
      //Serial.println(F("%"));
    }
        // Get humidity event and print its value.
		 }
  
  if (now - lastMoveMsg > 5000) {
    lastMoveMsg = now;
    if (digitalRead(pirPin) == LOW)
    {
      client.publish("movment", "0");
     // Serial.println("No motion");
    }
    else
    {
      client.publish("movment", "1");
      //Serial.println("Motion detected  ALARM");
    }
    
  }
}
